<?php
// $Id: d2c_analytics_session_stats.inc,v 1.1.2.3 2010/08/04 10:57:36 dot2code Exp $

/**
 *
 * @copyright 2010, (c) dot2code Technologies S.L.
 *
 */

/**
 * 
 * 
 */
class D2CAnalyticsSessionStats {
	
	/**
   * Constants for the different session status.
   */
	const StatusOK = 0;
	const StatusInvalidData = 1;
	const StatusInvalidTimestamp = 2;
  const	StatusInvalidAuthToken = 3;
  
  /**
   * Constants for the different known user agents.
   */
  const UserAgentUnknown = 0;
  const UserAgentChrome = 1;
  const UserAgentFF = 2;
  const UserAgentIE7 = 3;
  const UserAgentIE8 = 4;
  const UserAgentSafari = 5;
  
  public $session_id;
  public $session_status;
  public $auth_status;
  public $timestamp;
  public $last_timestamp;
  public $hits;
  public $uid;
  public $ip;
  public $user_agent;
  public $locale;
  public $code_local_id;
  public $interface;
  public $user_profile;
  public $op_login;
  public $op_register;
  public $op_unregister;
  public $processed;
  
  private $new_record;
  
  /**
   *  Class methods
   */
  
  /**
	 * Finds a certain D2C analytics session stats object in the local database.
	 * 
	 * @param $session_id
	 *   An integer with the session_id for the target object. 
	 *   
	 * @return
	 *   A D2CAnalyticsSessionStats instance if the object was found. Null, otherwise.
	 */
  public static function find($session_id) {
  	$result = db_query("SELECT * FROM {d2c_analytics_session_stats} WHERE session_id = %d", $session_id);
  	if ($result) {
  	  $attributes = db_fetch_array($result);
  	  if ($attributes) {
  	  	return new D2CAnalyticsSessionStats($attributes);
  	  }
  	}
		return NULL;
  }
  
  /**
   * Searches for D2CAnalyticSessionStats that fit to the given restrictions
   * 
   * @param $processed
   *   A boolean indicating if searching for processed stats (TRUE) or unprocessed stats (FALSE).
   * @param $window_start
   *   (optional) A timestamp. No stat logged before it will be returned.
   * @param $window_end
   *   (optional) A timestamp. No stat logged after it will be returned.
   * @param $limit
   *   (optional) A integer to determine the limit of rows to return.
   *   
   * @return
   *   An array of D2CAnalyticSessionStats. A single D2CAnalyticSessionStats instance if $limit = 1 (and
   *   the search returned a row).
   */
  public static function filter($processed, $window_start = NULL, $window_end = NULL, $limit = NULL) {
  	$db_query = "SELECT * FROM {d2c_analytics_session_stats} WHERE processed = %d";
  	$db_query_attributes[] = $processed;
  	if ($window_start) {
  		$db_query .= " AND timestamp >= %d";
  		$db_query_attributes[] = $window_start;
  	}
  	if ($window_end) {
  		$db_query .= " AND timestamp <= %d";
  		$db_query_attributes[] = $window_end; 
  	}
  	if ($limit) {
  		$db_query .= " LIMIT %d";
  		$db_query_attributes[] = $limit;
  	}
  	$result = db_query($db_query, $db_query_attributes); 	
  	if ($result) {
  		while ($row = db_fetch_array($result)) {
  		  if ($row['user_profile']) {
  			  $row['user_profile'] = unserialize($row['user_profile']);
  			}
  			$rows[] = $row;
  		}
  		if ($limit == 1) {
    	  return $rows[0];
  		} else {
  			return $rows;
  		}
  	}
  	else {
  	  return array();
  	}
  }
  
  /**
   * Sets the processed mark to all D2CAnalyticsSessionStats between two timstamps.
   * 
   * @param $window_start
   *   Start timestamp.
   * @param $window_end
   *   End timestamp.
   */
  public static function set_as_processed($window_start, $window_end) {
  	$db_query = "UPDATE {d2c_analytics_session_stats} SET processed = 1 WHERE processed = 0 AND timestamp >= %d AND timestamp <= %d";
  	db_query($db_query, $window_start, $window_end);
  }
  
  /**
   * Checks if a certain D2C QR code has its statistics enabled.
   * 
   * @param $code_id
   *   (optional) The local id for the D2C QR code. If not supplied, the default value for a new D2C QR code will be returned.
   *   
   * @return
   *   A boolean indicating if the statistics are enabled or not.
   */
  public static function enabled_for_code($code_id = NULL) {
  	if ($code_id) {
  	  $result = db_query("SELECT * FROM {d2c_analytics_enabled_codes} WHERE code_local_id = %d", $code_id);
  	  if ($result && ($attributes = db_fetch_array($result))) {
  		  return $attributes['enabled'];
  	  }
  	}
  	return TRUE;
  }
  
  /**
   * Enables / Disables statistics for a certain D2C QR code.
   * 
   * @param $code_id
   *   The local id for the D2C QR code.
   * @param $enabled
   *   A boolean indicating if the statistics must be enabled or not for this D2C QR code.
   */
  public static function set_enabled_for_code($code_id, $enabled) {
  	$result = db_query("SELECT * FROM {d2c_analytics_enabled_codes} WHERE code_local_id = %d", $code_id);
  	if($result && ($attributes = db_fetch_array($result))) {
  		db_query("UPDATE {d2c_analytics_enabled_codes} SET `enabled` = %d WHERE `code_local_id` = %d", $enabled, $code_id);
  	}
  	else {
  		db_query("INSERT INTO {d2c_analytics_enabled_codes} (`code_local_id`, `enabled`) VALUES (%d, %d)", $code_id, $enabled);
  	}
  }
  
  /**
   * Deletes the enabled / disabled statistics information for a certain D2C QR code.
   * 
   * @param $code_id
   *   The local id for the D2C QR code.
   */
  public static function delete_enabled_for_code($code_id) {
  	db_query("DELETE FROM {d2c_analytics_enabled_codes} WHERE code_local_id = %d", $code_id);
  }
  
  /**
   * Process a user agent string and returns a constant representing the browser used.
   * 
   * @param $user_agent
   *   A user agent string.
   *   
   * @return
   *   One of the constants defined in this class representing a browser.
   */
  public static function process_user_agent($user_agent) {
 	
  	switch(TRUE) {
  		case preg_match("/chrome/i", $user_agent):
  			return D2CAnalyticsSessionStats::UserAgentChrome;
  		case preg_match("/firefox/i", $user_agent):
  			return D2CAnalyticsSessionStats::UserAgentFF;
  		case preg_match("/msie 7/i", $user_agent):
  			return D2CAnalyticsSessionStats::UserAgentIE7;
  	  case preg_match("/msie 8/i", $user_agent):
  			return D2CAnalyticsSessionStats::UserAgentIE8;
  	  case preg_match("/applewebkit/i", $user_agent):
  			return D2CAnalyticsSessionStats::UserAgentSafari;
  		default:
  			return D2CAnalyticsSessionStats::UserAgentUnknown;
  	}
  	
  }
  
  /**
   * Class constructor.
   *
   * Default values will be given to all attributes unless explicit values are provided for them.
   *
   * @param $attributes
   *   (optional) An associative array with values for one or more of the D2C QR code's attributes. 
   */
  public function __construct($attributes = array()) {
  	if ($attributes['session_id']) {
  		$this->new_record = FALSE;
  	}
  	else {
  		$this->new_record = TRUE;
  		$this->session_id = time() + rand(100, 999) * 1000000000;
  	}
    $this->set_defaults();
    $this->set_attributes($attributes);
  }

  
  /**
   *  Instance methods
   */
  
  /**
   * Sets values for all provided attributes.
   * 
   * Unkown attributes will be ignored.
   * 
   * @param $attributes
   *   An associative array with values for one or more of the object's attributes.
   */
  public function set_attributes($attributes) {
    foreach($attributes as $attr => $value) {
    	try {
    		// Behaviour params data is unserialized if needed.
    	  if (($attr == 'user_profile') and is_string($value)) {
  	  	  $value = unserialize($value);
  	    }
    		$this->$attr = $value;
    	}
    	catch (Exception $e) {}
    }
  }
  
  /**
   * Saves the instance to the local database. This can be both an INSERT (for new instances) or an UPDATE.
   * 
   * @return
   *   Failure to write a record will return FALSE. Otherwise SAVED_NEW or SAVED_UPDATED is returned
   *   depending on the operation performed.
   */
  public function save() {
  	if ($this->new_record) {
  		return drupal_write_record('d2c_analytics_session_stats', &$this);
  	}
  	else {
  	  return d2c_content_write_record('d2c_analytics_session_stats', &$this, "session_id");
  	}
  }

  /**
   *  Private methods
   */
  
  /**
   * Sets default values for the attributes in this instance.
   */
  private function set_defaults() {
    $this->session_status = D2CAnalyticsSessionStats::StatusOK;
    $this->timestamp = time();
    $this->last_timestamp = time();
    $this->hits = 0;
    $this->op_login = FALSE;
    $this->op_register = FALSE;
    $this->op_unregister = FALSE;
    $this->processed = FALSE;
  }
  
}