<?php
// $Id: d2c_code.inc,v 1.1.2.3 2010/08/04 10:57:36 dot2code Exp $

/**
 *
 * @copyright 2010, (c) dot2code Technologies S.L.
 *
 */

module_load_include('inc', 'd2c_core', 'd2c_api');
module_load_include('inc', 'd2c_core', 'd2c_content_write_record');

/**
 * Instances of this class will represent a D2C QR code.
 * 
 * Internal state will include both administrative information (with relevance only locally, like a
 * textual description or a local identifier to do local searches) and a copy of the configuration
 * information about the mapped remote D2C QR code, if any (the same information that is keeped in
 * dot2code unless the local data has become unsynchronised).
 */
class D2CCode {
	
	/**
   * Constants for the different D2C QR code types.
   */
	const NodeRedirectionBehaviour = 0;
	const URLRedirectionBehaviour = 1;
  const	SendFileBehaviour = 2;
  const	VCardBehaviour = 3;
  const	CustomBehaviour = 4;
  
  const UploadedFileDirname = 'uploads';
  const CodeFilesDirname = 'codes';
  
	
	/**
   * @var A local identifier for the current D2C QR code.
   */
	public $local_id;
	
	/**
   * @var The name of the current D2C QR code. Just for administrative purposes.
   */
	public $name;
	
	/**
	 * @var A local description for the D2C QR code. Just for administrative purposes. 
	 */
	public $description;
		
	/**
   * @var D2C QR code's real identifier.
   */
	public $id;
	
	/**
   * @var D2C QR code's type as documented in dot2code. This will decide the kind of access control
   * that will be executed in dot2code for all incoming users.
   */
	public $code_type;
	
	/**
	 * @var Whether this D2C QR code is locally active or not (only active D2C QR codes will let users
	 * access the application).
	 */
	public $active;
	
	/**
	 * @var D2C QR code's kind of local behaviour that will be executed when a user accesses the domain.
	 */
	public $behaviour;
	
	/**
	 * @var D2C QR code's kind of local behaviour that will be executed when a user accesses the domain.
	 */
  public $behaviour_params;
  
  /**
   * @var Reference to a D2C campaign to which this D2C QR code has been asigned, if any.
   */
  public $campaign_id;
  
  /**
   * @var Reference to the D2C domain in which this D2C QR code has been created.
   */
  public $domain_id;
  
  /**
   * @var Creation timestamp.
   */
  public $created_at;
	
	
	/**
   *  Class methods
   */
  
  public static function uploaded_files_path() {
  	d2c_settings_get_files_path('uploaded');
  }
	
	/**
	 * Returns the list of D2C codes in the local database.
	 * 
	 * @return
	 *   An array of D2CCode instances.
	 */
  public static function all() {
  	$codes = array();
  	$result = db_query("SELECT * FROM {d2c_code} ORDER BY created_at DESC");
  	if ($result) {
      while ($code = db_fetch_array($result)) {
    	  $codes[] = new D2CCode($code);
      }
  	}
  	return $codes;
  }
  
  /**
	 * Finds D2C codes using their local ids in the local database.
	 * 
	 * @param $local_ids
	 *   An array of integers with the local ids for the target D2C QR codes, or a simple integer if only searching for one. 
	 *   
	 * @return
	 *   An array of D2CCode instances for all codes found. If searched for a single code, a D2CCode instance will be returned
	 *   if it has been found. Null will be returned otherwise.
	 */
  public static function find($local_ids) {
  	$codes = array();
  	if (!empty($local_ids)) {
  	  $result = db_query('SELECT * FROM {d2c_code} WHERE local_id IN (' . db_placeholders($local_ids) . ') ORDER BY created_at DESC', $local_ids);
	    if ($result) {
		    while($attributes = db_fetch_array($result)) {
			    $codes[$attributes['local_id']] = new D2CCode($attributes);
 		    }
 	    }	
  	}	  
    if (is_array($local_ids)) {
  	  return $codes;
    }
    elseif (count($codes) == 1) {
 	    return reset($codes);
 	  }
 	  else {
 	    return NULL;
 	  }
		
  }
  
  /**
	 * Returns the list of D2C codes in the local database, filtering by D2C domain and/or D2C campaign.
	 * 
	 * @param $domain_id
	 *   (optional)
	 * @param $campaign_id
	 *   (optional)
	 * @param $behaviour
	 *   (optional)
	 * @param $code_type
	 *   (optional)
	 * @param $page
	 * @param $per_page
	 * 
	 * @return
	 *   An array of D2CCode instances.
	 */
  public static function filter($domain_id = NULL, $campaign_id = NULL, $name = NULL, $behaviour = NULL, $code_type = NULL, $active = NULL, $page = 0, $per_page = 10) {
  	$codes = array();
  	$query = "SELECT * FROM {d2c_code}";
  	$conditions = array();
  	$args = array();
    if (!is_null($name) and !($name == '')) {
  	  $conditions[] = "name LIKE '%%%s%%'";
  	  $args[] = $name;
  	}
  	if ($domain_id) {
  	  $conditions[] = "domain_id = %d";
  	  $args[] = $domain_id;
  	}
  	if (!is_null($campaign_id) and !($campaign_id == '')) {
  		if ($campaign_id) {
  			$conditions[] = "campaign_id = %d";
  			$args[] = $campaign_id;
  	  } else {
  	    $conditions[] = "(campaign_id IS NULL or campaign_id = '')";
  	  }
  	}
  	if (!is_null($behaviour) and !($behaviour == '')) {
  		$conditions[] = "behaviour = %d";
  		$args[] = $behaviour;
  	}
    if (!is_null($code_type) and !($code_type == '')) {
  		$conditions[] = "code_type = %d";
  		$args[] = $code_type;
  	}
  	if (!is_null($active) and !($active == '')) {
  	  $conditions[] = "active = %d";
  	  $args[] = $active;
  	}
  	if (count($conditions) > 0) {
  		$query .= " WHERE " . implode(' AND ', $conditions);
  	}
  	$query .= " ORDER BY created_at DESC";
  	$result = db_query_range($query, $args, ($page * $per_page), $per_page);
  	if ($result) {
      while ($code = db_fetch_array($result)) {
    	  $codes[] = new D2CCode($code);
      }
  	}
  	return $codes;
  }
  
  /**
   * Deletes a D2C code, both in dot2code's servers and in the local database.
   * 
   * @param $local_id
   *   An integer with the local id of the D2C QR code to delete.
   */
  public static function delete($local_id) {
  	if ($code = D2CCode::find($local_id)) {
  		$code->destroy();
  	}
  }
  
  /**
   * Returns the list of possible behaviours for a D2C QR code.
   * 
   * @return
   *   An associative array with all possible behaviours a D2C QR code may
   *   have and their human-readable names.
   */
  public static function behaviours() {
  	return array(
  	  D2CCode::NodeRedirectionBehaviour => t('Node redirection'),
  	  D2CCode::URLRedirectionBehaviour => t('URL redirection'),
  	  D2CCode::SendFileBehaviour => t('Send file'),
  	  D2CCode::VCardBehaviour => t('VCard'),
  	  D2CCode::CustomBehaviour => t('Custom'),
  	);
  }
	
  
  /**
   *  Instance methods
   */
  
  /**
   * Class constructor.
   *
   * Default values will be given to all attributes unless explicit values are provided for them.
   *
   * @param $attributes
   *   (optional) An associative array with values for one or more of the D2C QR code's attributes. 
   */
  public function __construct($attributes = array()) {
    $this->set_defaults();
    $this->set_attributes($attributes);
  }
  
  /**
   * Sets values for all provided attributes.
   * 
   * Unkown attributes will be ignored.
   * 
   * @param $attributes
   *   An associative array with values for one or more of the D2C QR code's attributes.
   */
  public function set_attributes($attributes) {
    foreach($attributes as $attr => $value) {
    	try {
    		// Behaviour params data is unserialized if needed.
    	  if (($attr == 'behaviour_params') and is_string($value)) {
  	  	  $value = unserialize($value);
  	    }
    		// Changing the behaviour params and a file was already defined for SendFileBehaviour.
    		if ($attr == 'behaviour_params' && $this->behaviour_params && $this->behaviour_params['fid']) {
    			// fid = 0: Old file is preserved. Otherwise, the old file is deleted.
    			if ($value['fid'] == 0) {
    				$value['fid'] = $this->behaviour_params['fid'];
    			} elseif ($old_file = d2c_find_file($this->behaviour_params['fid'])) {
    			  db_query('DELETE FROM {files} WHERE fid = %d', $old_file['fid']);
            // We must be careful. If a new file was given and it has replaced the old one, a delete operation is not needed.
            $new_file = d2c_find_file($value['fid']);
            if (!$new_file or ($new_file['filepath'] != $old_file['filepath'])) {
              file_delete($old_file['filepath']);
    		    }    			  
    			}
    		}
    		$this->$attr = $value;
    	}
    	catch (Exception $e) {}
    }
  }
  
  /**
   * Returns all values in this instance for the list of attributes returned by function attribute_names().
   * 
   * @return
   *   An associative array with attribute names as keys and their current values.
   */
  public function attributes() {
  	$attributes = array();
  	foreach(D2CCode::attribute_names() as $attribute) {
  		$attributes[$attribute] = $this->$attribute;
  	}
  	return $attributes;
  }
  
  /**
   * Saves the instance to the local database. This can be both an INSERT (for new instances) or an UPDATE.
   * 
   * @return
   *   Failure to write a record will return FALSE. Otherwise SAVED_NEW or SAVED_UPDATED is returned
   *   depending on the operation performed.
   */
  public function save() {
  	if ($this->local_id) {
  		return d2c_content_write_record('d2c_code', &$this, "local_id");
  	}
  	else {
  	  return drupal_write_record('d2c_code', &$this);
  	}
  }
  
  /**
   * Sends all current D2C QR code's data to dot2code.
   * 
   * This can be both a D2C QR code creation operation (for new instances) or a D2C QR code edit operation (for
   * already existent D2C QR codes in dot2code). 
   */
  public function remote_save() {
  	$domain = $this->domain();
  	if($domain) {
  	  if ($this->id) {
  	  	// @todo D2C QR codes edition
  	    // $api = new D2CAPI($domain->id, $domain->secret_key);
  	    // $this->set_attributes($api->edit_code($this->attributes()));
  	  }
  	  else {
        $api = new D2CAPI($domain->id, $domain->secret_key);
        // We do a first save to the database to get a proper local_id.
        $this->save();
        try {
          $this->set_attributes($api->create_qr_code($this->code_type, array('local_id' => $this->local_id), TRUE));
        } catch (D2CTransportException $e) {
        	$this->local_destroy();
        	throw $e;
        }
      }
      $this->save();
  	}
  }
  
  /**
   * Locally activates this D2C QR code.
   */
  public function activate() {
  	$this->active = true;
  	$this->save();
  }
  
  /**
   * Locally deactivates this D2C QR code.
   */
  public function deactivate() {
  	$this->active = false;
  	$this->save();
  }
  
  /**
   * Deletes this D2C QR code both locally and in dot2code's servers.
   */
  public function destroy() {
  	$domain = $this->domain();
  	if($domain) {
  		$api = new D2CAPI($domain->id, $domain->secret_key);
  		$api->delete_qr_code($this->id);
  	}
  	$this->local_destroy();
  }
  
  /**
   * Local deletion of the D2C QR code.
   */
  public function local_destroy() {
  	module_invoke_all('d2c_core_delete_code_hook', $this->local_id);
  	db_query("DELETE FROM {d2c_code} WHERE local_id = %d", $this->local_id);
    // D2C QR code image is deleted.
  	if ($image = $this->image()) {
  		db_query('DELETE FROM {files} WHERE filepath = "%s"', $image);
  		file_delete($image);
  	}
  	// Any attached file is also deleted.
  	if ($this->behaviour_params and ($uploaded_file = d2c_find_file($this->behaviour_params['fid']))) {
  		db_query('DELETE FROM {files} WHERE fid = "%d"', $uploaded_file['fid']);
  		file_delete($uploaded_file['filepath']);
  	}
  }
  
  /**
   * D2C campaign to which this D2C QR code has been asigned, if any.
   * 
   * @return
   *   An instance of D2CCampaign or NULL if this D2C QR Code has not been assigned
   *   to any D2C campaign.
   */
  public function campaign() {
    if ($this->campaign_id) {
  	  return D2CCampaign::find($this->campaign_id);
  	}
  	else {
  	  return NULL;
  	}
  }

  /**
   * D2C domain in which this D2C QR code has been created.
   * 
   * @return
   *   An instance of D2CDomain or NULL if the D2C domain
   *   hasn't been yet assigned.
   */
  public function domain() {
  	if ($this->domain_id) {
  	  return D2CDomain::find($this->domain_id);
  	}
  	else {
  	  return NULL;
  	}
  }
  
  public function image() {
  	global $user;
  	
  	$dir = d2c_settings_get_files_path(D2CCode::CodeFilesDirname);
  	$file_name = $this->local_id . '.png';
  	$file_path = "$dir/" . $file_name;
  	if (is_file($file_path)) {
  		return $file_path;
  	}
  	else {
  		try {
  			$domain = $this->domain();
  			$api = new D2CAPI($domain->id, $domain->secret_key);  			
  			$image = $api->get_qr_code_image($this->id, 'png');
  			if (imagepng($image, $file_path)) {
  				// Any old entry in the files table for this D2C QR code image is removed to avoid duplicates.
  				db_query('DELETE FROM {files} WHERE filepath = "%s"', $file_path);
  				$file = new stdClass();
  				$file->uid = $user->uid;
          $file->filename = $file_name;
          $file->filepath = $file_path;
          $file->filemime = file_get_mimetype($file_name);
          $file->filesize = filesize($file_path);
          $file->status = FILE_STATUS_PERMANENT;
          $file->timestamp = time();
          drupal_write_record('files', $file);
  				return $file_path;
  			}
  		}
  		catch(D2CTransportException $e) {
  		}
  		return NULL;
  	}
  }
  
  /**
   *  Private methods
   */
  
  /**
   * Sets default values for the attributes in this instance.
   */
  private function set_defaults() {
  	$this->code_type = D2CAPI::CODE_TYPE_REUSABLE;
  	$this->active = TRUE;
  	$default_domain = D2CDomain::default_domain();
  	if ($default_domain) {
  	  $this->domain_id = $default_domain->local_id;
  	}
  }
  
  /**
   * Returns the list of D2C domain attributes that dot2code can understand.
   * 
   * @return
   *   An array with the name of all attributes used in remote operations.
   */
  private static function attribute_names() {
  	return array('id', 'code_type'); // @todo extra_data
  }
	
}

?>
